@@ -33,56 +33,71 @@ class Project(object):
             self._init(path_to_tx)
 
     def _init(self, path_to_tx=None):
-        # The path to the root of the project, where .tx lives!
-        self.root = path_to_tx or find_dot_tx()
-        logger.debug("Path to tx is %s." % self.root)
-        if not self.root:
-            MSG("Cannot find any .tx directory!")
-            MSG("Run 'tx init' to initialize your project first!")
-            raise ProjectNotInit()
-
-        # The path to the config file (.tx/config)
-        self.config_file = os.path.join(self.root, ".tx", "config")
-        logger.debug("Config file is %s" % self.config_file)
-        # Touch the file if it doesn't exist
-        if not os.path.exists(self.config_file):
-            MSG("Cannot find the config file (.tx/config)!")
-            MSG("Run 'tx init' to fix this!")
-            raise ProjectNotInit()
-
-        # The dictionary which holds the config parameters after deser/tion.
-        # Read the config in memory
-        self.config = OrderedRawConfigParser()
+        instructions = "Run 'tx init' to initialize your project first!"
         try:
-            self.config.read(self.config_file)
+            self.root = self._get_tx_dir_path(path_to_tx)
+            self.config_file = self._get_config_file_path(self.root)
+            self.config = self._read_config_file(self.config_file)
+            self.txrc_file = self._get_transifex_file()
+            self.txrc = self._get_transifex_config(self.txrc_file)
+        except ProjectNotInit, e:
+            logger.error('\n'.join([unicode(e), instructions]))
+            raise
+
+    def _get_config_file_path(self, root_path):
+        """Check the .tx/config file exists."""
+        config_file = os.path.join(root_path, ".tx", "config")
+        logger.debug("Config file is %s" % config_file)
+        if not os.path.exists(config_file):
+            msg = "Cannot find the config file (.tx/config)!"
+            raise ProjectNotInit(msg)
+        return config_file
+
+    def _get_tx_dir_path(self, path_to_tx):
+        """Check the .tx directory exists."""
+        root_path = path_to_tx or find_dot_tx()
+        logger.debug("Path to tx is %s." % root_path)
+        if not root_path:
+            msg = "Cannot find any .tx directory!"
+            raise ProjectNotInit(msg)
+        return root_path
+
+    def _read_config_file(self, config_file):
+        """Parse the config file and return its contents."""
+        config = OrderedRawConfigParser()
+        try:
+            config.read(config_file)
         except Exception, err:
-            MSG("WARNING: Cannot open/parse .tx/config file", err)
-            MSG("Run 'tx init' to fix this!")
-            raise ProjectNotInit()
+            msg = "Cannot open/parse .tx/config file: %s" % err
+            raise ProjectNotInit(msg)
+        return config
 
+    def _get_transifex_config(self, txrc_file):
+        """Read the configuration from the .transifexrc file."""
+        txrc = OrderedRawConfigParser()
+        try:
+            txrc.read(txrc_file)
+        except Exception, e:
+            msg = "Cannot read global configuration file: %s" % e
+            raise ProjectNotInit(msg)
+        return txrc
 
-        home = os.path.expanduser("~")
-        self.txrc_file = os.path.join(home, ".transifexrc")
-        logger.debug(".transifexrc file is at %s" % home)
-        if not os.path.exists(self.txrc_file):
-            MSG("No configuration file found.")
-            # Writing global configuration file
+    def _get_transifex_file(self, directory=None):
+        """Fetch the path of the .transifexrc file.
+
+        It is in the home directory ofthe user by default.
+        """
+        if directory is None:
+            directory = os.path.expanduser('~')
+        txrc_file = os.path.join(directory, ".transifexrc")
+        logger.debug(".transifexrc file is at %s" % directory)
+        if not os.path.exists(txrc_file):
+            msg = "No authentication data found."
+            logger.info(msg)
             mask = os.umask(077)
-            open(self.txrc_file, 'w').close()
+            open(txrc_file, 'w').close()
             os.umask(mask)
-
-        self.txrc = OrderedRawConfigParser()
-        try:
-            self.txrc.read(self.txrc_file)
-        except Exception, err:
-            MSG("WARNING: Cannot global conf file (%s)" %  err)
-            MSG("Run 'tx init' to fix this!")
-            raise ProjectNotInit()
-
-
-    def create_resource(self):
-        pass
-
+        return txrc_file
 
     def validate_config(self):
         """
@@ -99,7 +114,7 @@ class Project(object):
             username = self.txrc.get(host, 'username')
             passwd = self.txrc.get(host, 'password')
         except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
-            MSG("No entry found for host %s. Creating..." % host)
+            logger.info("No entry found for host %s. Creating..." % host)
             username = user or raw_input("Please enter your transifex username: ")
             while (not username):
                 username = raw_input("Please enter your transifex username: ")
@@ -107,7 +122,7 @@ class Project(object):
             while (not passwd):
                 passwd = getpass.getpass()
 
-            MSG("Updating %s file..." % self.txrc_file)
+            logger.info("Updating %s file..." % self.txrc_file)
             self.txrc.add_section(host)
             self.txrc.set(host, 'username', username)
             self.txrc.set(host, 'password', passwd)
@@ -288,13 +303,20 @@ class Project(object):
             return os.path.join(self.root, relpath)
 
     def pull(self, languages=[], resources=[], overwrite=True, fetchall=False,
-        fetchsource=False, force=False, skip=False, minimum_perc=0):
-        """
-        Pull all translations file from transifex server
-        """
+        fetchsource=False, force=False, skip=False, minimum_perc=0, mode=None):
+        """Pull all translations file from transifex server."""
         self.minimum_perc = minimum_perc
         resource_list = self.get_chosen_resources(resources)
 
+        if mode == 'reviewed':
+            url = 'pull_reviewed_file'
+        elif mode == 'translator':
+            url = 'pull_translator_file'
+        elif mode == 'developer':
+            url = 'pull_developer_file'
+        else:
+            url = 'pull_file'
+
         for resource in resource_list:
             logger.debug("Handling resource %s" % resource)
             self.resource = resource
@@ -305,6 +327,8 @@ class Project(object):
             lang_map = self.get_resource_lang_mapping(resource)
             host = self.get_resource_host(resource)
             logger.debug("Language mapping is: %s" % lang_map)
+            if mode is None:
+                mode = self._get_option(resource, 'mode')
             self.url_info = {
                 'host': host,
                 'project': project_slug,
@@ -314,6 +338,7 @@ class Project(object):
 
             stats = self._get_stats_for_resource()
 
+
             try:
                 file_filter = self.config.get(resource, 'file_filter')
             except ConfigParser.NoOptionError:
@@ -328,8 +353,8 @@ class Project(object):
                     files, slang, lang_map, stats, force
                 )
                 if new_translations:
-                    MSG("New translations found for the following languages: %s" %
-                        ', '.join(new_translations))
+                    msg = "New translations found for the following languages: %s"
+                    logger.info(msg % ', '.join(new_translations))
 
             existing, new = self._languages_to_pull(
                 languages, files, lang_map, stats, force
@@ -346,8 +371,8 @@ class Project(object):
 
             if pull_languages:
                 logger.debug("Pulling languages for: %s" % pull_languages)
-                MSG("Pulling translations for resource %s (source: %s)" %
-                    (resource, sfile))
+                msg = "Pulling translations for resource %s (source: %s)"
+                logger.info(msg % (resource, sfile))
 
             for lang in pull_languages:
                 local_lang = lang
@@ -369,22 +394,27 @@ class Project(object):
                     'stats': stats,
                     'local_file': local_file,
                     'force': force,
+                    'mode': mode,
                 }
                 if not self._should_update_translation(**kwargs):
                     msg = "Skipping '%s' translation (file: %s)."
-                    MSG(msg % (color_text(remote_lang, "RED"), local_file))
+                    logger.info(
+                        msg % (color_text(remote_lang, "RED"), local_file)
+                    )
                     continue
 
                 if not overwrite:
                     local_file = ("%s.new" % local_file)
-                MSG(" -> %s: %s" % (color_text(remote_lang,"RED"), local_file))
+                logger.warning(
+                    " -> %s: %s" % (color_text(remote_lang, "RED"), local_file)
+                )
                 try:
-                    r = self.do_url_request('pull_file', language=remote_lang)
+                    r = self.do_url_request(url, language=remote_lang)
                 except Exception,e:
                     if not skip:
                         raise e
                     else:
-                        ERRMSG(e)
+                        logger.error(e)
                         continue
                 base_dir = os.path.split(local_file)[0]
                 mkdir_p(base_dir)
@@ -393,8 +423,8 @@ class Project(object):
                 fd.close()
 
             if new_translations:
-                MSG("Pulling new translations for resource %s (source: %s)" %
-                (resource, sfile))
+                msg = "Pulling new translations for resource %s (source: %s)"
+                logger.info(msg % (resource, sfile))
                 for lang in new_translations:
                     if lang in lang_map.keys():
                         local_lang = lang_map[lang]
@@ -411,8 +441,18 @@ class Project(object):
                         local_file = relpath(os.path.join(trans_dir, '%s_translation' %
                             local_lang, os.curdir))
 
-                    MSG(" -> %s: %s" % (color_text(remote_lang, "RED"), local_file))
-                    r = self.do_url_request('pull_file', language=remote_lang)
+                    if lang != slang:
+                        satisfies_min = self._satisfies_min_translated(
+                            stats[remote_lang], mode
+                        )
+                        if not satisfies_min:
+                            msg = "Skipping language %s due to used options."
+                            logger.info(msg % lang)
+                            continue
+                    logger.warning(
+                        " -> %s: %s" % (color_text(remote_lang, "RED"), local_file)
+                    )
+                    r = self.do_url_request(url, language=remote_lang)
 
                     base_dir = os.path.split(local_file)[0]
                     mkdir_p(base_dir)
@@ -426,6 +466,8 @@ class Project(object):
         Push all the resources
         """
         resource_list = self.get_chosen_resources(resources)
+        self.skip = skip
+        self.force = force
         for resource in resource_list:
             push_languages = []
             project_slug, resource_slug = resource.split('.')
@@ -442,7 +484,7 @@ class Project(object):
                 'resource': resource_slug
             }
 
-            MSG("Pushing translations for resource %s:" % resource)
+            logger.info("Pushing translations for resource %s:" % resource)
 
             stats = self._get_stats_for_resource()
 
@@ -457,14 +499,14 @@ class Project(object):
 
             if source:
                 if sfile == None:
-                    ERRMSG("You don't seem to have a proper source file"
+                    logger.error("You don't seem to have a proper source file"
                         " mapping for resource %s. Try without the --source"
                         " option or set a source file first and then try again." %
                         resource)
                     continue
                 # Push source file
                 try:
-                    MSG("Pushing source file (%s)" % sfile)
+                    logger.warning("Pushing source file (%s)" % sfile)
                     if not self._resource_exists(stats):
                         logger.info("Resource does not exist.  Creating...")
                         fileinfo = "%s;%s" % (resource_slug, slang)
@@ -481,7 +523,7 @@ class Project(object):
                     if not skip:
                         raise e
                     else:
-                        ERRMSG(e)
+                        logger.error(e)
             else:
                 try:
                     self.do_url_request('resource_details')
@@ -504,8 +546,8 @@ class Project(object):
                             l = lang_map[l]
                         push_languages.append(l)
                         if l not in f_langs:
-                            ERRMSG("Warning: No mapping found for language code '%s'." %
-                                color_text(l,"RED"))
+                            msg = "Warning: No mapping found for language code '%s'."
+                            logger.error(msg % color_text(l,"RED"))
                 logger.debug("Languages to push are %s" % push_languages)
 
                 # Push translation files one by one
@@ -526,10 +568,13 @@ class Project(object):
                     }
                     if not self._should_push_translation(**kwargs):
                         msg = "Skipping '%s' translation (file: %s)."
-                        MSG(msg % (color_text(lang, "RED"), local_file))
+                        logger.info(msg % (color_text(lang, "RED"), local_file))
                         continue
 
-                    MSG("Pushing '%s' translations (file: %s)" % (color_text(remote_lang, "RED"), local_file))
+                    msg = "Pushing '%s' translations (file: %s)"
+                    logger.warning(
+                         msg % (color_text(remote_lang, "RED"), local_file)
+                    )
                     try:
                         self.do_url_request(
                             'push_translation', multipart=True, method='PUT',
@@ -543,69 +588,115 @@ class Project(object):
                         if not skip:
                             raise e
                         else:
-                            ERRMSG(e)
+                            logger.error(e)
 
     def delete(self, resources=[], languages=[], skip=False, force=False):
         """Delete translations."""
         resource_list = self.get_chosen_resources(resources)
-        for resource in resource_list:
-            delete_languages = []
-            files = self.get_resource_files(resource)
-            project_slug, resource_slug = resource.split('.')
-            lang_map = self.get_resource_lang_mapping(resource)
-            host = self.get_resource_host(resource)
+        self.skip = skip
+        self.force = force
 
+        if not languages:
+            delete_func = self._delete_resource
+        else:
+            delete_func = self._delete_translations
+
+        for resource in resource_list:
+            project_slug, resource_slug = resource.split('.')
+            host = self.get_resource_host(resource)
             self.url_info = {
                 'host': host,
                 'project': project_slug,
                 'resource': resource_slug
             }
-
-            project_details = parse_json(self.do_url_request('project_details',
-                project = self))
+            logger.debug("URL data are: %s" % self.url_info)
+            project_details = parse_json(
+                self.do_url_request('project_details', project=self)
+            )
             teams = project_details['teams']
             stats = self._get_stats_for_resource()
+            delete_func(project_details, resource, stats, languages)
 
-            logger.debug("URL data are: %s" % self.url_info)
-
-            MSG("Deleting translations from resource %s:" % resource)
-            if not languages:
-                logger.warning("No languages specified.")
-                return
-            for language in languages:
-                if language not in stats:
-                    if not skip:
-                        msg = "Skipping: %s : Translation does not exist."
-                        MSG(msg % (language))
+    def _delete_resource(self, project_details, resource, stats, *args):
+        """Delete a resource from Transifex."""
+        project_slug, resource_slug = resource.split('.')
+        project_resource_slugs = [
+            r['slug'] for r in project_details['resources']
+        ]
+        logger.info("Deleting resource %s:" % resource)
+        if resource_slug not in project_resource_slugs:
+            if not self.skip:
+                msg = "Skipping: %s : Resource does not exist."
+                logger.info(msg % resource)
+            return
+        if not self.force:
+            slang = self.get_resource_option(resource, 'source_lang')
+            for language in stats:
+                if language == slang:
                     continue
-                if not force:
-                    if language in teams:
-                        msg = "Skipping: %s : Unable to delete translation "\
-                            "because it is associated with a team.\n"\
-                            "Please use -f or --force option to delete "\
-                            "this translation."
-                        MSG(msg % language)
-                        continue
-                    if int(stats[language]['translated_entities']) > 0:
-                        msg = "Skipping: %s : Unable to delete translation "\
-                            "because it is not empty.\n"\
-                            "Please use -f or --force option to delete this "\
-                            "translation."
-                        MSG(msg % language)
-                        continue
-
-                try:
-                    self.do_url_request(
-                        'delete_translation', language=language, method="DELETE"
+                if int(stats[language]['translated_entities']) > 0:
+                    msg = (
+                        "Skipping: %s : Unable to delete resource because it "
+                        "has a not empty %s translation.\nPlease use -f or "
+                        "--force option to delete this resource."
                     )
+                    logger.info(msg % (resource, language))
+                    return
+        try:
+            self.do_url_request('delete_resource', method="DELETE")
+            self.config.remove_section(resource)
+            self.save()
+            msg = "Deleted resource %s of project %s."
+            logger.info(msg % (resource_slug, project_slug))
+        except Exception, e:
+            msg = "Unable to delete resource %s of project %s."
+            logger.error(msg % (resource_slug, project_slug))
+            if not self.skip:
+                raise
 
-                    msg = "Deleted language %s from resource %s in project %s."
-                    MSG(msg % (language, resource_slug, project_slug))
-                except Exception, e:
-                    msg = "ERROR: Unable to delete translation %s"
-                    MSG(msg % language)
-                    if not skip:
-                        raise
+    def _delete_translations(self, project_details, resource, stats, languages):
+        """Delete the specified translations for the specified resource."""
+        logger.info("Deleting translations from resource %s:" % resource)
+        for language in languages:
+            self._delete_translation(project_details, resource, stats, language)
+
+    def _delete_translation(self, project_details, resource, stats, language):
+        """Delete a specific translation from the specified resource."""
+        project_slug, resource_slug = resource.split('.')
+        if language not in stats:
+            if not self.skip:
+                msg = "Skipping %s: Translation does not exist."
+                logger.warning(msg % (language))
+            return
+        if not self.force:
+            teams = project_details['teams']
+            if language in teams:
+                msg = (
+                    "Skipping %s: Unable to delete translation because it is "
+                    "associated with a team.\nPlease use -f or --force option "
+                    "to delete this translation."
+                )
+                logger.warning(msg % language)
+                return
+            if int(stats[language]['translated_entities']) > 0:
+                msg = (
+                    "Skipping %s: Unable to delete translation because it "
+                    "is not empty.\nPlease use -f or --force option to delete "
+                    "this translation."
+                )
+                logger.warning(msg % language)
+                return
+        try:
+            self.do_url_request(
+                'delete_translation', language=language, method="DELETE"
+            )
+            msg = "Deleted language %s from resource %s of project %s."
+            logger.info(msg % (language, resource_slug, project_slug))
+        except Exception, e:
+            msg = "Unable to delete translation %s"
+            logger.error(msg % language)
+            if not self.skip:
+                raise
 
     def do_url_request(self, api_call, multipart=False, data=None,
                        files=[], encoding=None, method="GET", **kwargs):
@@ -628,6 +719,7 @@ class Project(object):
         kwargs['hostname'] = hostname
         kwargs.update(self.url_info)
         url = (API_URLS[api_call] % kwargs).encode('UTF-8')
+        logger.debug(url)
 
         opener = None
         headers = None
@@ -653,9 +745,14 @@ class Project(object):
 
         try:
             fh = urllib2.urlopen(req)
+            raw = fh.read()
+            fh.close()
+            return raw
         except urllib2.HTTPError, e:
             if e.code in [401, 403, 404]:
                 raise e
+            elif 200 <= e.code < 300:
+                return None
             else:
                 # For other requests, we should print the message as well
                 raise Exception("Remote server replied: %s" % e.read())
@@ -663,12 +760,9 @@ class Project(object):
             error = e.args[0]
             raise Exception("Remote server replied: %s" % error[1])
 
-        raw = fh.read()
-        fh.close()
-        return raw
 
-
-    def _should_update_translation(self, lang, stats, local_file, force=False):
+    def _should_update_translation(self, lang, stats, local_file, force=False,
+                                   mode=None):
         """Whether a translation should be udpated from Transifex.
 
         We use the following criteria for that:
@@ -682,12 +776,13 @@ class Project(object):
             stats: The (global) statistics object.
             local_file: The local translation file.
             force: A boolean flag.
+            mode: The mode for the translation.
         Returns:
             True or False.
         """
         return self._should_download(lang, stats, local_file, force)
 
-    def _should_add_translation(self, lang, stats, force=False):
+    def _should_add_translation(self, lang, stats, force=False, mode=None):
         """Whether a translation should be added from Transifex.
 
         We use the following criteria for that:
@@ -699,12 +794,14 @@ class Project(object):
             lang: The language code to check.
             stats: The (global) statistics object.
             force: A boolean flag.
+            mode: The mode for the translation.
         Returns:
             True or False.
         """
         return self._should_download(lang, stats, None, force)
 
-    def _should_download(self, lang, stats, local_file=None, force=False):
+    def _should_download(self, lang, stats, local_file=None, force=False,
+                         mode=None):
         """Return whether a translation should be downloaded.
 
         If local_file is None, skip the timestamps check (the file does
@@ -716,7 +813,7 @@ class Project(object):
             logger.debug("No lang %s in statistics" % lang)
             return False
 
-        satisfies_min = self._satisfies_min_translated(lang_stats)
+        satisfies_min = self._satisfies_min_translated(lang_stats, mode)
         if not satisfies_min:
             return False
 
@@ -792,7 +889,7 @@ class Project(object):
             return None
         return time.mktime(time.gmtime(os.path.getmtime(path)))
 
-    def _satisfies_min_translated(self, stats):
+    def _satisfies_min_translated(self, stats, mode=None):
         """Check whether a translation fulfills the filter used for
         minimum translated percentage.
 
@@ -801,7 +898,7 @@ class Project(object):
         Returns:
             True or False
         """
-        cur = self._extract_completed(stats)
+        cur = self._extract_completed(stats, mode)
         option_name = 'minimum_perc'
         if self.minimum_perc is not None:
             minimum_percent = self.minimum_perc
@@ -842,16 +939,21 @@ class Project(object):
         return True
 
     @classmethod
-    def _extract_completed(cls, stats):
+    def _extract_completed(cls, stats, mode=None):
         """Extract the information for the translated percentage from the stats.
 
         Args:
             stats: The stats object for a language as returned by Transifex.
+            mode: The mode of translations requested.
         Returns:
             The percentage of translation as integer.
         """
+        if mode == 'reviewed':
+            key = 'reviewed_percentage'
+        else:
+            key = 'completed'
         try:
-            return int(stats['completed'][:-1])
+            return int(stats[key][:-1])
         except KeyError, e:
             return 0
 
@@ -972,7 +1074,7 @@ class Project(object):
             res = parse_json(self.do_url_request('formats'))
             return res[i18n_type]['file-extensions'].split(',')[0]
         except Exception,e:
-            ERRMSG(e)
+            logger.error(e)
             return ''
 
     def _resource_exists(self, stats):
@@ -1024,8 +1126,8 @@ class Project(object):
         i18n_type = self._get_option(resource, 'type')
         if i18n_type is None:
             logger.error(
-                "Cannot get type for resource. Please, add it in "
-                "the .tx/config file."
+                "Please define the resource type in .tx/config (eg. type = PO)."
+                " More info: http://bit.ly/txcl-rt"
             )
 
         opener = urllib2.build_opener(MultipartPostHandler)
@@ -1076,3 +1178,26 @@ class Project(object):
                 return self.config.get('main', option)
         return value
 
+    def set_i18n_type(self, resources, i18n_type):
+        """Set the type for the specified resources."""
+        self._set_resource_option(resources, key='type', value=i18n_type)
+
+    def set_min_perc(self, resources, perc):
+        """Set the minimum percentage for the resources."""
+        self._set_resource_option(resources, key='minimum_perc', value=perc)
+
+    def set_default_mode(self, resources, mode):
+        """Set the default mode for the specified resources."""
+        self._set_resource_option(resources, key='mode', value=mode)
+
+    def _set_resource_option(self, resources, key, value):
+        """Set options in the config file.
+
+        If resources is empty. set the option globally.
+        """
+        if not resources:
+            self.config.set('main', key, value)
+            return
+        for r in resources:
+            self.config.set(r, key, value)
+
