@@ -7,20 +7,7 @@ import urllib2 # This should go and instead use do_url_request everywhere
 
 from urls import API_URLS
 from txclib.log import logger
-
-
-def MSG(msg, verbose=1):
-    """
-    STDOUT logging function
-    """
-    logger.info('%s' % msg)
-
-
-def ERRMSG(msg, verbosity=1):
-    """
-    STDERR logging function
-    """
-    logger.error('%s' % msg)
+from txclib.exceptions import UnknownCommandError
 
 
 def find_dot_tx(path = os.path.curdir, previous = None):
@@ -74,7 +61,7 @@ def parse_tx_url(url):
         if m:
             return type, m.groupdict()
 
-    raise Exception("tx: Malformed url given!")
+    raise Exception("tx: Malformed url given. Please refer to our docs: http://bit.ly/txautor")
 
 
 def get_details(api_call, username, password, *args, **kwargs):
@@ -144,10 +131,6 @@ def discover_commands():
     return command_table
 
 
-class UnknownCommandError(Exception):
-	pass
-
-
 def exec_command(command, *args, **kwargs):
     """
     Execute given command
